# -*- encoding: utf-8 -*-
'''
@File        :bez.py
@Time        :2021/09/28 11:26:28
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :None
'''

import configparser
import os
import re
import math
from math import *
import numpy as np
import pandas as pd
import CentriVision
from Bio import Seq, SeqIO, SeqRecord
import codecs
from tqdm import trange
import gc
import matplotlib.pyplot as plt
from matplotlib.patches import *
from matplotlib.patches import Circle, Ellipse
from pylab import *
from collections import Counter

def config():
    conf = configparser.ConfigParser()
    conf.read(os.path.join(CentriVision.__path__[0], 'conf.ini'))
    return conf.items('ini')

def load_conf(file, section):
    conf = configparser.ConfigParser()
    conf.read(file)
    return conf.items(section)






# -*- encoding: utf-8 -*-
'''
@File        :run.py
@Time        :2021/09/28 09:04:51
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :主程序
'''


import argparse
import os
import sys
import configparser
import pandas as pd
import CentriVision
import CentriVision.bez as bez


from CentriVision.getPASA import getPASA
from CentriVision.getNCBI import getNCBI



parser = argparse.ArgumentParser(
    prog = 'CentriVision', usage = '%(prog)s [options]', epilog = "", formatter_class = argparse.RawDescriptionHelpFormatter,)
parser.description = '''\
runing CentriVision
    -------------------------------------- '''
parser.add_argument("-v", "--version", action = 'version', version='0.0.0')



parser.add_argument("-getpasa", dest = "getPASA",
                    help = "get_PASA")
parser.add_argument("-getncbi", dest = "getNCBI",
                    help = "get_NCBI")

args = parser.parse_args()



def run_getPASA():
    options = bez.load_conf(args.getPASA, 'getPASA')
    getPASA1 = getPASA(options)
    getPASA1.run()

def run_getNCBI():
    options = bez.load_conf(args.getNCBI, 'getNCBI')
    getNCBI1 = getNCBI(options)
    getNCBI1.run()


def module_to_run(argument):
    switcher = {


        'getPASA': run_getPASA,
        'getNCBI': run_getNCBI,


    }
    return switcher.get(argument)()

def main():
    path = CentriVision.__path__[0]
    options = {

               'getPASA': 'getPASA.conf',
               'getNCBI': 'getNCBI.conf',

               }
    for arg in vars(args):
        value = getattr(args, arg)
        # print(value)
        if value is not None:
            if value in ['?', 'help', 'example']:
                f = open(os.path.join(path, 'example', options[arg]))
                print(f.read())
            elif value == 'e':
                out = '''\
        File example
        [fpchrolen]
        chromosomes number_of_bases
        *   *
        *   *
        *   *
        [fpgff]
        chromosomes gene    start   end
        *   *   *   *
        *   *   *   *
        *   *   *   *
        [fpgenefamilyinf]
        gene1   gene2   Ka  Ks
        *   *   *   *
        *   *   *   *
        *   *   *   *
        [alphagenepairs]
        gene1   gene2
        *   *   *
        *   *   *
        *   *   *

        The file columns are separated by Tab
        -----------------------------------------------------------    '''
                print(out)
            elif not os.path.exists(value):
                print(value+' not exits')
                sys.exit(0)
            else:
                module_to_run(arg)




# python
# -*- encoding: utf-8 -*-
'''
@File        :getPASA.py
@Time        :2021/04/27 19:54:08
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :python getPASA.py oldgff oldpep oldcds oldgene genome xxx
@annotation        :
'''

import sys
import os
import gc# 内存管理模块
from tqdm import trange
from Bio.Seq import Seq
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
import numpy as np
import pandas as pd
import CentriVision.bez as bez

class getPASA():
    def __init__(self, options):
        self.gene_r8_Separator1 = ';' # 基因注释外部分隔符
        self.gene_r8_Separator2 = '=' # 基因注释名与注释值分隔符
        self.gene_row_key = 'ID'
        self.mrna_gene_key = 'Parent'
        self.mrna_row_key = 'ID'
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def read_r8(self,s):
        dic = {}
        lt = s.split(self.gene_r8_Separator1)
        for i in lt:
            if self.gene_r8_Separator2 in i:
                lt0 = i.split(self.gene_r8_Separator2)
                dic[lt0[0]] = lt0[1]
        return dic

    def getgene_r8(self,s,types):
        dic = {}
        lt = s.split(self.gene_r8_Separator1)
        for i in lt:
            if self.gene_r8_Separator2 in i:
                lt0 = i.split(self.gene_r8_Separator2)
                dic[lt0[0]] = lt0[1]
        if types == 'gene':
            return dic['ID']
        elif types == 'mRNA':
            return dic['Parent']
        else:
            return dic['Parent'].replace("model", "TU")

    def getgene_r81(self,s,types):
        dic = {}
        lt = s.split(self.gene_r8_Separator1)
        for i in lt:
            if self.gene_r8_Separator2 in i:
                lt0 = i.split(self.gene_r8_Separator2)
                dic[lt0[0]] = lt0[1]
        if types == 'gene':
            return dic['ID']
        elif types == 'mRNA':
            return dic['Parent']
        else:
            return dic['Parent']

    def ncbi_gff(self):
        # if os.path.exists(self.species+'_gene.gff'):
        #     os.remove(self.species+'_gene.gff')
        # if os.path.exists(self.species+'_mrna.gff'):
        #     os.remove(self.species+'_mrna.gff')
        # if os.path.exists(self.species+'.cds'):
        #     os.remove(self.species+'.cds')
        # if os.path.exists(self.species+'.pep'):
        #     os.remove(self.species+'.pep')
        # if os.path.exists(self.species+'.gene.fa'):
        #     os.remove(self.species+'.gene.fa')
        # if os.path.exists(self.species+'_cds.gff3'):
        #     os.remove(self.species+'_cds.gff3')
        # if os.path.exists(self.species+'_exon.gff3'):
        #     os.remove(self.species+'_exon.gff3')
        # if os.path.exists(self.species+'.lens'):
        #     os.remove(self.species+'.lens')

        genome = self.readncbigenome()
        pepdoc,pepc = self.readncbipep()
        cdsdoc,cdsc = self.readncbicds()
        genedoc = self.readncbigene(list(pepdoc.keys()),cdsc)

        lensf = open(self.species+'.lens','w')
        pepf = open(self.species+'.pep','w')
        cdsf = open(self.species+'.cds','w')
        genef = open(self.species+'.gene.fa','w')
        gffout = open(self.species+'_gene.gff','w')
        cdsgffout = open(self.species+'_cds.gff3','w')
        exongffout = open(self.species+'_exon.gff3','w')

        gff = pd.read_csv(self.gff3_file,header = None, sep='\t', comment='#')
        gff[9]=gff.apply(lambda x:self.getgene_r8(x[8],x[2]),axis=1)
        chrsgff = gff.groupby(0).groups
        for key in chrsgff.keys():# 遍历染色体
            print(" ******** ",key," ******** ")
            local = gff.loc[chrsgff[key]]
            local.reset_index(drop=True,inplace=True)
            print(local)
            chr_gene_dic0 = local.groupby(9).groups# 按照第几列分组
            genegff = local.groupby(2).groups# 按照第几列分组
            genepd = local.loc[genegff['gene']].sort_values(by=[0,3],ascending= [True,True])
            genepd.reset_index(drop=True,inplace=True)
            print(genepd,'\n',len(genepd))
            lens = [key,str(len(genome[key].seq)),str(len(genepd))]
            lensf.write('\t'.join(lens)+'\n')
            for index,row in genepd.iterrows():# 遍历基因
                name = row[0] + 'g' + str(index+1).zfill(4)
                gene = local.loc[chr_gene_dic0[row[9]]]
                gene[10] = gene.apply(lambda x:self.getgene_r81(x[8],x[2]),axis=1)
                kripke = gene.groupby(2).groups# 按照第几列分组
                mRNA = gene.loc[kripke['mRNA']].sort_values(by=[0,3],ascending= [True,True])
                mRNA.reset_index(drop=True,inplace=True)
                oldid = mRNA.iloc[0, 9]
                pep_seq,cds_seq,gene_seq = '','',''
                pepname = ''
                for index1,row in mRNA.iterrows():# 遍历基因的mrna结构
                    dic = self.read_r8(row[8])
                    if dic['ID'] in pepdoc.keys():
                        pep_seq = pepdoc[dic['ID']]
                        cds_seq = cdsdoc[dic['ID']]
                        pepname = dic['ID']
                        break
                if pepname == '':
                    for index1,row in mRNA.iterrows():# 遍历基因的mrna结构
                        pep_seq = pepdoc[pepc[row[9]]]
                        cds_seq = cdsdoc[pepc[row[9]]]
                        pepname = pepc[row[9]]
                        break
                if pepname == '':
                    print(oldid,pepname,' 基因不存在？')
                    exit()

                try:
                    gene_seq = genedoc[pepname]
                except:
                    print(pepname,' 这个基因没有找到gene序列')
                    exit()

                cds_s = SeqRecord(Seq(str(cds_seq.seq)),id=name,name=name,description=oldid)
                SeqIO.write(cds_s, cdsf, "fasta")
                gene_s = SeqRecord(Seq(str(gene_seq.seq)),id=name,name=name,description=oldid)
                SeqIO.write(gene_s, genef, "fasta")
                pep_s = SeqRecord(Seq(str(pep_seq.seq)),id=name,name=name,description=oldid)
                SeqIO.write(pep_s, pepf, "fasta")
                genegff = [row[0],name,str(row[3]),str(row[4]),str(row[6]),str(index+1),oldid]
                gffout.write('\t'.join(genegff)+'\n')
                genecds = gene.drop(gene[(gene[2] != 'CDS') | (gene[10] != pepname)].index)
                for index1,row in genecds.iterrows():
                    cdsgff = [row[0],name,'CDS',str(row[3]),str(row[4]),str(row[5]),str(row[6]),str(row[7]),row[9],row[10]]
                    cdsgffout.write('\t'.join(cdsgff)+'\n')
                geneexon = gene.drop(gene[(gene[2] != 'exon') | (gene[10] != pepname)].index)
                for index1,row in geneexon.iterrows():
                    exonff = [row[0],name,'exon',str(row[3]),str(row[4]),str(row[5]),str(row[6]),str(row[7]),row[9],row[10]]
                    exongffout.write('\t'.join(exonff)+'\n')

        pepf.close()
        cdsf.close()
        genef.close()
        lensf.close()
        gffout.close()
        cdsgffout.close()
        exongffout.close()

    def readncbipep(self):
        print('reading ncbi-pep')
        pep,pepc = {},{}
        for seq_record in SeqIO.parse(self.pep_file, "fasta"):# biopython循环读取
            gene = str(seq_record.description).split()[2]
            if gene not in pepc.keys():
                pepc[gene] = seq_record.id
                pep[seq_record.id] = seq_record
            else:
                if len(seq_record.seq) > len(pep[pepc[gene]].seq):
                    del pep[pepc[gene]]
                    pepc[gene] = seq_record.id
                    pep[pepc[gene]] = seq_record
                elif len(seq_record.seq) == len(pep[pepc[gene]].seq):
                    if seq_record.id < pepc[gene]:
                        del pep[pepc[gene]]
                        pepc[gene] = seq_record.id
                        pep[pepc[gene]] = seq_record
                    else:
                        pass
                else:
                    pass
        print('PEP of number is ',len(pep))
        return pep,pepc

    def readncbicds(self):
        print('reading ncbi-cds')
        cds,cdsc = {},{}
        for seq_record in SeqIO.parse(self.cds_file, "fasta"):# biopython循环读取
            gene = str(seq_record.description).split()[2]
            if gene not in cdsc.keys():
                cdsc[gene] = seq_record.id
                cds[seq_record.id] = seq_record
            else:
                if len(seq_record.seq) > len(cds[cdsc[gene]].seq):
                    del cds[cdsc[gene]]
                    cdsc[gene] = seq_record.id
                    cds[cdsc[gene]] = seq_record
                elif len(seq_record.seq) == len(cds[cdsc[gene]].seq):
                    if seq_record.id < cdsc[gene]:
                        del cds[cdsc[gene]]
                        cdsc[gene] = seq_record.id
                        cds[cdsc[gene]] = seq_record
                    else:
                        pass
                else:
                    pass
        print('CDS of number is ',len(cds))
        return cds,cdsc


    def readncbigene(self,pep,cdsc):
        print('reading ncbi-gene')
        cds = {}
        for seq_record in SeqIO.parse(self.gene_file, "fasta"):# biopython循环读取
            gene = str(seq_record.description).split()[2]
            if seq_record.id in pep:
                cds[seq_record.id] = seq_record
            else:
                cds[cdsc[gene]] = seq_record
        print('gene of number is ',len(cds))
        return cds

    def readncbigenome(self):
        print('reading genome')
        genome = SeqIO.to_dict(SeqIO.parse(self.genome_file, "fasta"))# 提取之后直接返回字典
        print('Chr of number is ',len(genome))
        return genome

    def run(self):
        self.ncbi_gff()



# python
# -*- encoding: utf-8 -*-
'''
@File        :bio.py
@Time        :2021/04/27 19:54:08
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :python bio.py gff pep cds ncbi/embl xxx
@annotation        :
'''

import sys
import ast
import os
import gc# 内存管理模块
from tqdm import trange
from Bio.Seq import Seq
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord

import numpy as np
import pandas as pd
import CentriVision.bez as bez

class getNCBI():
    def __init__(self, options):
        self.gene_r8_Separator1 = ';' # 基因注释外部分隔符
        self.gene_r8_Separator2 = '=' # 基因注释名与注释值分隔符
        self.gene_row_key = 'ID'
        self.mrna_gene_key = 'Parent'
        self.mrna_row_key = 'ID'
        # self.gff3_file = 'x.gff3'
        # self.pep_file = 'x.gff3'
        # self.cds_file = 'x.gff3'
        # self.species = 'x.gff3'
        self.gene_file = 'None'
        self.genome_file = 'None'
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def read_r8(self,s):
        dic = {}
        lt = s.split(self.gene_r8_Separator1)
        for i in lt:
            if self.gene_r8_Separator2 in i:
                lt0 = i.split(self.gene_r8_Separator2)
                dic[lt0[0]] = lt0[1]
        return dic

    def get_chr(self,chr0,maker,index):
        return self.species + chr0.split(maker)[index]
        # return self.species + chr0[3:]

    def ncbi_gff(self):
        if os.path.exists(self.species+'_gene.gff'):
            os.remove(self.species+'_gene.gff')
        if os.path.exists(self.species+'_mrna.gff'):
            os.remove(self.species+'_mrna.gff')
        if os.path.exists(self.species+'.cds'):
            os.remove(self.species+'.cds')
        if os.path.exists(self.species+'.pep'):
            os.remove(self.species+'.pep')


        gff = pd.read_csv(self.gff3_file,header = None, sep='\t', comment='#')
        print(gff)
        print("************************************************** 注意 **************************************************")
        a = input("如果染色体名称需要稍作修改(EVM_01 -> at01,默认修改为EVM_01 -> atEVM_01)请选择Y,不需要则输入N or enter:\n")
        if a == "Y":
            maker = input("请输入染色体ID的分割符号：")
            index = input("请输入染色体ID分割后保留序号：")
            print("将输出如下ID:",self.species+str(gff.iloc[0, 0]).split(maker)[int(index)])
            gff[0] = gff.apply(lambda row: self.get_chr(row[0],maker,int(index)), axis = 1)
            print(gff)
            if self.genome_file != "None":
                a = input("若基因组染色体名称需要稍作修改(EVM_01 -> at01,默认修改为EVM_01 -> atEVM_01)请选择Y,不需要则输入N or enter:\n")
                if a != "Y":
                    genome = self.readncbigenome0()
                else:
                    genome = self.readncbigenome(self.genome_file,maker,int(index))
            else:
                print("缺失genome文件之后lens文件染色体长度默认为染色体末位基因结尾\n请注意！！！")
        else:
            #gff[0] = gff.apply(lambda row: self.get_chr(row[0],' ',0), axis = 1)
            if self.genome_file != "None":
                a = input("若基因组染色体名称需要稍作修改(EVM_01 -> at01,默认修改为EVM_01 -> atEVM_01)请选择Y,不需要则输入N or enter:\n")
                if a != "Y":
                    genome = self.readncbigenome0()
                else:
                    genome = self.readncbigenome(self.genome_file,maker,int(index))
                # genome = self.readncbigenome0()
                #genome = self.readncbigenome(self.genome_file,' ',0)
            else:
                print("缺失genome文件之后lens文件染色体长度默认为染色体末位基因结尾\n请注意！！！")

        dic = gff.groupby(2).groups# 按照第几列分组
        gene = gff.loc[dic['gene']].sort_values(by=[0,3],ascending= [True,True])
        gene.reset_index(drop=True,inplace=True)
        # print(gene)

        # 基因分染色体
        lens = open(self.species+".lens",'w')
        dic0 = gene.groupby(0).groups
        gene_dic = {}
        for key in dic0.keys():
            print(" ******** ",key," ******** ")
            local = gene.loc[dic0[key]].sort_values(by=[0,3],ascending= [True,True])
            local.reset_index(drop=True,inplace=True)
            print(local)
            for index,row in local.iterrows():
                name = row[0] + 'g' + str(index+1).zfill(4)
                local.loc[index,1] = name
                old_name = self.read_r8(row[8])[self.gene_row_key]
                local.loc[index,8] = old_name
                local.loc[index,7] = index+1
                gene_dic[old_name] = name
            row_1 = local.iloc[-1].tolist()
            if self.genome_file != "None":
                #print(genome.keys())
                lens_s = [key,str(len(genome[key].seq)),str(int(row_1[7]))]
            else:
                lens_s = [key,str(row_1[4]),str(int(row_1[7]))]
            lens.write('\t'.join(lens_s)+'\n')
            local = local[[0,1,3,4,6,7,8]]
            local.to_csv(self.species+'_gene.gff', index=False, header=False,sep='\t',mode='a+')
            print(local)
        del dic0,local

        pepdoc = self.readncbipep(self.pep_file,' ',0)
        cdsdoc = self.readncbicds(self.cds_file,' ',0)
        print("\n************************************************** 注意 **************************************************")
        mrna = gff.loc[dic['mRNA']].sort_values(by=[0,3],ascending= [True,True])
        mrna.reset_index(drop=True,inplace=True)
        first_row = mrna.iloc[0]
        print(first_row)
        print('self.mrna_row_key ',self.mrna_row_key)
        print('pep file ID ',list(pepdoc.keys())[0])
        print('cds file ID ',list(cdsdoc.keys())[0])
        print('If the protein ID and CDS ID formats are inconsistent, the code needs to be modified.')

        a = input('请输入需要修改的文件 C/P/CP or None\n')
        if a == 'C' or a.upper() == 'C':
            print('cds file ID ',list(cdsdoc.keys())[0])
            maker = input("请输入CDS序列ID的分割符号：")
            index = input("请输入CDS序列ID分割后保留序号：")
            print("将输出如下ID:",list(cdsdoc.keys())[0].split(maker)[int(index)])
            cdsdoc = self.readncbicds(self.cds_file,maker,int(index))
        elif a == 'P' or a.upper() == 'P':
            print('pep file ID ',list(pepdoc.keys())[0])
            maker = input("请输入PEP序列ID的分割符号：")
            index = input("请输入PEP序列ID分割后保留序号：")
            print("将输出如下ID:",list(pepdoc.keys())[0].split(maker)[int(index)])
            pepdoc = self.readncbipep(self.pep_file,maker,int(index))
        elif a == 'CP' or a.upper() == 'CP':
            print('cds file ID ',list(cdsdoc.keys())[0])
            print('pep file ID ',list(pepdoc.keys())[0])
            maker = input("请输入CDS序列ID的分割符号：")
            index = input("请输入CDS序列ID分割后保留序号：")
            print("将输出如下ID:",list(cdsdoc.keys())[0].split(maker)[int(index)])
            cdsdoc = self.readncbicds(self.cds_file,maker,int(index))
            maker = input("请输入PEP序列ID的分割符号：")
            index = input("请输入PEP序列ID分割后保留序号：")
            print("将输出如下ID:",list(pepdoc.keys())[0].split(maker)[int(index)])
            pepdoc = self.readncbipep(self.pep_file,maker,int(index))
        elif a == 'None' or a == '':
            pass
        else:
            pass

        pep = open(self.species+'.pep','w')
        cds = open(self.species+'.cds','w')
        pepdoc1,cdsdoc1 = {},{}
        
        
        dic0 = mrna.groupby(0).groups
        # mrna_dic = {}
        for key in dic0.keys():
            print(" ******** ",key," ******** ")
            local = mrna.loc[dic0[key]].sort_values(by=[0,3],ascending= [True,True])
            local.reset_index(drop=True,inplace=True)
            print(local)
            for index,row in local.iterrows():
                gene_name = self.read_r8(row[8])[self.mrna_gene_key]
                tran_name = self.read_r8(row[8])[self.mrna_row_key]
                name = gene_dic[gene_name]
                local.loc[index,1] = name
                local.loc[index,8] = gene_name
                # gene_dic[old_name] = name
                if name not in cdsdoc1.keys() and tran_name in cdsdoc.keys() and tran_name in pepdoc.keys():
                    cdsdoc1[name] = SeqRecord(Seq(str(cdsdoc[tran_name].seq)),id=name,name=name,description=gene_name)
                    pepdoc1[name] = SeqRecord(Seq(str(pepdoc[tran_name].seq)),id=name,name=name,description=gene_name)
                elif name in cdsdoc1.keys() and tran_name in cdsdoc.keys() and tran_name in pepdoc.keys():
                    if len(str(cdsdoc[tran_name].seq)) > len(str(cdsdoc1[name].seq)):
                        cdsdoc1[name] = SeqRecord(Seq(str(cdsdoc[tran_name].seq)),id=name,name=name,description=gene_name)
                        pepdoc1[name] = SeqRecord(Seq(str(pepdoc[tran_name].seq)),id=name,name=name,description=gene_name)
                    else:
                        continue
            local = local[[0,1,2,3,4,6,7,8]]
            local.reset_index(drop=True,inplace=True)
            local[2] = 'mRNA'
            local.to_csv(self.species+'_mrna.gff', index=False, header=False,sep='\t',mode='a+')
            print(local)
        for key in cdsdoc1.keys():
            SeqIO.write(cdsdoc1[key], cds, "fasta")
            SeqIO.write(pepdoc1[key], pep, "fasta")

        pep.close()
        cds.close()


    def genenum(self,gfflist):
        num = 0# 计数
        for gff in gfflist:
            if gff[2] == 'gene':
                num += 1
            else:
                pass
        k = len(str(num))
        return k

    def readncbipep(self,file,maker,index):
        print('reading ncbi-pep')
        pep = {}
        for seq_record in SeqIO.parse(file, "fasta"):# biopython循环读取
            gene = str(seq_record.id).split(maker)[index]
            pep[gene] = seq_record
        print('pep of number is ',len(pep))
        return pep

    def readncbicds(self,file,maker,index):
        print('reading ncbi-cds')
        cds = {}
        for seq_record in SeqIO.parse(file, "fasta"):# biopython循环读取
            gene = str(seq_record.id).split(maker)[index]
            cds[gene] = seq_record
        print('cds of number is ',len(cds))
        return cds

    def readncbigenome(self,file,maker,index):
        print('reading ncbi-genome')
        genome = {}
        for seq_record in SeqIO.parse(file, "fasta"):# biopython循环读取
            gene = self.species + str(seq_record.id).split(maker)[index]
            genome[gene] = seq_record
        print('genome of number is ',len(genome))
        return genome

    def readncbigenome0(self):
        print('reading genome')
        genome = SeqIO.to_dict(SeqIO.parse(self.genome_file, "fasta"))# 提取之后直接返回字典
        print('Chr of number is ',len(genome))
        return genome

    def readncbipep0(self):
        print('reading ncbi-pep')
        pep = SeqIO.to_dict(SeqIO.parse(self.pep_file, "fasta"))# 提取之后直接返回字典
        print('PEP of number is ',len(pep))
        return pep

    def readncbigene0(self):
        print('reading ncbi-gene')
        cds = {}
        for seq_record in SeqIO.parse(sys.argv[2], "fasta"):# biopython循环读取
            gene = str(seq_record.id).split('CDS')[0][:-1]
            cds[gene] = seq_record
        print('gene of number is ',len(cds))
        return cds

    def readncbicds0(self):
        print('reading ncbi-cds')
        cds = SeqIO.to_dict(SeqIO.parse(self.cds_file, "fasta"))# 提取之后直接返回字典
        print('CDS of number is ',len(cds))
        return cds

    def sequence(self,chrlist):
        newchr = {}
        doc = {}
        for i in chrlist:
            doc[str(i)] = int(i[1])
        dic1SortList = sorted(doc.items(),key = lambda x:x[1],reverse = True)
        for j in range(len(dic1SortList)):
            lt0 = ast.literal_eval(dic1SortList[j][0])
            lt0.append(j + 1)
            newchr[lt0[0]] = lt0
        return newchr

    def filewrite(self,file,name,seq):
        f = open(file,'a+',encoding = 'utf-8')
        f.write('>' + name + '\n')
        f.write(seq)
        f.close()

    def writegff(self,file,lt):
        f = open(file,'a+',encoding = 'utf-8')
        f.write(str(self.species) + str(lt[0]) + '\t' + str(lt[1]) + '\t' \
            + str(lt[2]) + '\t' + str(lt[3]) + '\t' \
            + str(lt[4]) + '\t' +str(lt[5]) + '\t' \
            +str(lt[6]) + '\n')
        f.close()

    def writelens(self,file,lt):
        f = open(file,'a+',encoding = 'utf-8')
        f.write(str(self.species) + str(lt[0]) + '\t' + str(lt[1]) + '\t' \
            + str(lt[2]) + '\n')
        f.close()

    def runncbi(self):
        self.ncbi_gff()
        exit()

        pepdoc = self.readncbipep()
        cdsdoc = self.readncbicds()
        genenum = self.genenum(gfflist)
        chrdoc1 = self.sequence(chrlist)
        genenums = 0
        print('gff文件筛选及cds、pep生成')
        for i in trange(len(gfflist)):
            gffrow = gfflist[i]
            if gffrow[2] == 'mRNA':
                gene = str(gffrow[8]['gene'])## 注意
                if gene in cdsdoc.keys() and str(cdsdoc[gene][0]) in pepdoc.keys():
                    genenums += 1
                    ol_chr = str(gffrow[0])
                    if len(str(chrdoc1[ol_chr][0])) <= 10 and str(chrdoc1[ol_chr][0]) != 'Unknown':
                        newchrname = chrdoc1[ol_chr][-2]
                    else:
                        newchrname = chrdoc1[ol_chr][-1]
                    # 在这儿修改名字，需求按照染色体长度排序
                    newname = self.species + str(newchrname) + 'g' + str(genenums).zfill(genenum)
                    self.filewrite(self.species + '.pep',newname,pepdoc[str(cdsdoc[gene][0])])
                    self.filewrite(self.species + '.cds',newname,cdsdoc[gene][1])
                    lt0 = [gffrow[0],gffrow[3],gffrow[4],gffrow[6],str(gene),newchrname,newname]
                    self.gff.append(lt0)
                elif gene in cdsdoc.keys():
                    genenums += 1
                    ol_chr = str(gffrow[0])
                    if len(str(chrdoc1[ol_chr][0])) <= 10 and str(chrdoc1[ol_chr][0]) != 'Unknown':
                        newchrname = chrdoc1[ol_chr][-2]
                    else:
                        newchrname = chrdoc1[ol_chr][-1]
                    # 在这儿修改名字，需求按照染色体长度排序
                    newname = self.species + str(newchrname) + 'g' + str(genenums).zfill(genenum)
                    # self.filewrite(self.species + '.pep',newname,pepdoc[str(cdsdoc[gene][0])])
                    self.filewrite(self.species + '.cds',newname,cdsdoc[gene][1])
                    lt0 = [gffrow[0],gffrow[3],gffrow[4],gffrow[6],str(gene),newchrname,newname]
                    self.gff.append(lt0)
                else:
                    pass
        gc.collect()# 内存释放
        genenum1 = 1
        chrlist0 = []
        chr0 = []
        print('gff、lens生成')
        for i in trange(len(self.gff)):
            gene = self.gff[i]
            if gene[-2] not in chrlist0:
                if chr0 != []:
                    self.writelens(str(self.species) + '.lens', chr0)
                    chr0 = []
                chrlist0.append(gene[-2])
                genenum1 = 1
                lt = [gene[-2],gene[-1],gene[1],gene[2],gene[3],genenum1,gene[4]]
                self.writegff(str(self.species) + '.gff', lt)
            else:
                genenum1 += 1
                chr0 = []
                chr0 = chr0 + [gene[-2],chrdoc1[gene[0]][1],genenum1]
                lt = [gene[-2],gene[-1],gene[1],gene[2],gene[3],genenum1,gene[4]]
                self.writegff(str(self.species) + '.gff', lt)
        if chr0 != []:
            self.writelens(str(self.species) + '.lens', chr0)

    def run(self):
        self.runncbi()


# data = getNCBI()
# data.run()
